class UseCaseNotFoundException implements Exception {
  final Type useCaseType;

  UseCaseNotFoundException(this.useCaseType);
}

class UseCaseContainer {
  final _cache = <Type, Object>{};
  final _useCasesFactory = <Type, Object Function()>{};
  final _keepAliveMarkers = <Type>{};

  UseCaseContainer({
    Map<Type, Object Function()> useCasesFactory = const <Type, Object Function()>{},
    Set<Type> keepAliveUntilRemoveMarkers = const <Type>{},
  }) {
    update(
      useCasesFactory: useCasesFactory,
      keepAliveUntilRemoveMarkers: keepAliveUntilRemoveMarkers,
    );
  }

  void update({
    Map<Type, Object Function()> useCasesFactory = const <Type, Object Function()>{},
    Set<Type> keepAliveUntilRemoveMarkers = const <Type>{},
  }) {
    _useCasesFactory.addAll(useCasesFactory);
    _keepAliveMarkers.addAll(keepAliveUntilRemoveMarkers);
  }

  void clearCurrent<U>(){
    _cache.remove(U);
  }

  U useCase<U>() {
    if(_keepAliveMarkers.contains(U)){
      if(_cache[U] == null){
        _cache[U] = _useCasesFactory[U]!();
      }
      return _cache[U]! as U;
    }else{
      return _useCasesFactory[U]!() as U;
    }
  }

  void removeMarker<U>(){
    _keepAliveMarkers.remove(U);
  }

  void addMarker<U>(){
    _keepAliveMarkers.add(U);
  }

  void clear(){
    _cache.clear();
    _useCasesFactory.clear();
    _keepAliveMarkers.clear();
  }

}
